@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""A Mongo driver for Python."""
+"""Python driver for MongoDB."""
 
 import types
 import sys
@@ -33,7 +33,7 @@ SLOW_ONLY = 1
 ALL = 2
 """Profile all operations."""
 
-version = "1.1"
+version = "1.1.1"
 """Current version of PyMongo."""
 
 Connection = PyMongo_Connection
