@@ -602,92 +602,23 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         }
         return 1;
     } else if (PyObject_IsInstance(value, DBRef)) {
-        int start_position,
-            length_location,
-            collection_length,
-            type_pos,
-            length;
-        PyObject* collection_object;
-        PyObject* encoded_collection;
-        PyObject* id_object;
-        char zero = 0;
-
+        PyObject* as_doc = PyObject_CallMethod(value, "as_doc", NULL);
+        if (!as_doc) {
+            return 0;
+        }
+        if (!write_dict(buffer, as_doc, 0)) {
+            Py_DECREF(as_doc);
+            return 0;
+        }
+        Py_DECREF(as_doc);
         *(buffer->buffer + type_byte) = 0x03;
-        start_position = buffer->position;
-
-        /* save space for length */
-        length_location = buffer_save_bytes(buffer, 4);
-        if (length_location == -1) {
-            return 0;
-        }
-
-        collection_object = PyObject_GetAttrString(value, "collection");
-        if (!collection_object) {
-            return 0;
-        }
-        encoded_collection = PyUnicode_AsUTF8String(collection_object);
-        Py_DECREF(collection_object);
-        if (!encoded_collection) {
-            return 0;
-        }
-        {
-            const char* collection = PyString_AsString(encoded_collection);
-            if (!collection) {
-                Py_DECREF(encoded_collection);
-                return 0;
-            }
-            id_object = PyObject_GetAttrString(value, "id");
-            if (!id_object) {
-                Py_DECREF(encoded_collection);
-                return 0;
-            }
-
-            if (!buffer_write_bytes(buffer, "\x02$ref\x00", 6)) {
-                Py_DECREF(encoded_collection);
-                Py_DECREF(id_object);
-                return 0;
-            }
-            collection_length = strlen(collection) + 1;
-            if (!buffer_write_bytes(buffer, (const char*)&collection_length, 4)) {
-                Py_DECREF(encoded_collection);
-                Py_DECREF(id_object);
-                return 0;
-            }
-            if (!buffer_write_bytes(buffer, collection, collection_length)) {
-                Py_DECREF(encoded_collection);
-                Py_DECREF(id_object);
-                return 0;
-            }
-        }
-        Py_DECREF(encoded_collection);
-
-        type_pos = buffer_save_bytes(buffer, 1);
-        if (type_pos == -1) {
-            Py_DECREF(id_object);
-            return 0;
-        }
-        if (!buffer_write_bytes(buffer, "$id\x00", 4)) {
-            Py_DECREF(id_object);
-            return 0;
-        }
-        if (!write_element_to_buffer(buffer, type_pos, id_object, check_keys)) {
-            Py_DECREF(id_object);
-            return 0;
-        }
-        Py_DECREF(id_object);
-
-        /* write null byte and fill in length */
-        if (!buffer_write_bytes(buffer, &zero, 1)) {
-            return 0;
-        }
-        length = buffer->position - start_position;
-        memcpy(buffer->buffer + length_location, &length, 4);
         return 1;
     }
     else if (PyObject_HasAttrString(value, "pattern") &&
              PyObject_HasAttrString(value, "flags")) { /* TODO just a proxy for checking if it is a compiled re */
         PyObject* py_flags = PyObject_GetAttrString(value, "flags");
         PyObject* py_pattern;
+        PyObject* encoded_pattern;
         long int_flags;
         char flags[7];
         int pattern_length,
@@ -702,16 +633,27 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         if (!py_pattern) {
             return 0;
         }
+
+        if (PyUnicode_Check(py_pattern)) {
+            encoded_pattern = PyUnicode_AsUTF8String(py_pattern);
+            Py_DECREF(py_pattern);
+            if (!encoded_pattern) {
+                return 0;
+            }
+        } else {
+            encoded_pattern = py_pattern;
+        }
+
         {
-            const char* pattern =  PyString_AsString(py_pattern);
+            const char* pattern =  PyString_AsString(encoded_pattern);
             pattern_length = strlen(pattern) + 1;
 
             if (!buffer_write_bytes(buffer, pattern, pattern_length)) {
-                Py_DECREF(py_pattern);
+                Py_DECREF(encoded_pattern);
                 return 0;
             }
         }
-        Py_DECREF(py_pattern);
+        Py_DECREF(encoded_pattern);
 
         flags[0] = 0;
         /* TODO don't hardcode these */
@@ -834,7 +776,7 @@ static int write_pair(bson_buffer* buffer, const char* name, Py_ssize_t name_len
     int type_byte;
 
     /* Don't write any _id elements unless we're explicitly told to -
-     * _id has to be written first so we write do so, but don't bother
+     * _id has to be written first so we do so, but don't bother
      * deleting it from the dictionary being written. */
     if (!allow_id && strcmp(name, "_id") == 0) {
         return 1;
@@ -856,6 +798,37 @@ static int write_pair(bson_buffer* buffer, const char* name, Py_ssize_t name_len
     return 1;
 }
 
+static int decode_and_write_pair(bson_buffer* buffer, PyObject* key,
+                                 PyObject* value, unsigned char check_keys) {
+    PyObject* encoded;
+    if (PyUnicode_Check(key)) {
+        encoded = PyUnicode_AsUTF8String(key);
+        if (!encoded) {
+            return 0;
+        }
+    } else if (PyString_Check(key)) {
+        encoded = key;
+        Py_INCREF(encoded);
+    } else {
+        PyObject* errmsg = PyString_FromString("documents must have only string keys, key was ");
+        PyObject* repr = PyObject_Repr(key);
+        PyString_ConcatAndDel(&errmsg, repr);
+        PyErr_SetString(InvalidDocument, PyString_AsString(errmsg));
+        Py_DECREF(errmsg);
+        return 0;
+    }
+
+    /* Don't allow writing _id here - it was already written. */
+    if (!write_pair(buffer, PyString_AsString(encoded),
+                    PyString_Size(encoded), value, check_keys, 0)) {
+        Py_DECREF(encoded);
+        return 0;
+    }
+
+    Py_DECREF(encoded);
+    return 1;
+}
+
 static int write_son(bson_buffer* buffer, PyObject* dict, int start_position,
                      int length_location, unsigned char check_keys) {
     PyObject* keys = PyObject_CallMethod(dict, "keys", NULL);
@@ -868,7 +841,6 @@ static int write_son(bson_buffer* buffer, PyObject* dict, int start_position,
     for(i = 0; i < items; i++) {
         PyObject* key;
         PyObject* value;
-        PyObject* encoded;
 
         key = PyList_GetItem(keys, i);
         if (!key) {
@@ -876,27 +848,11 @@ static int write_son(bson_buffer* buffer, PyObject* dict, int start_position,
             return 0;
         }
         value = PyDict_GetItem(dict, key);
-        if (!value) {
+        if (!value ||
+            !decode_and_write_pair(buffer, key, value, check_keys)) {
             Py_DECREF(keys);
             return 0;
         }
-        if (PyUnicode_CheckExact(key)) {
-            encoded = PyUnicode_AsUTF8String(key);
-            if (!encoded) {
-                Py_DECREF(keys);
-                return 0;
-            }
-        } else {
-            encoded = key;
-            Py_INCREF(encoded);
-        }
-        /* Don't allow writing _id here - it was written above. */
-        if (!write_pair(buffer, PyString_AsString(encoded),
-                        PyString_Size(encoded), value, check_keys, 0)) {
-            Py_DECREF(keys);
-            Py_DECREF(encoded);
-            return 0;
-        }
     }
     Py_DECREF(keys);
     return 1;
@@ -938,20 +894,7 @@ static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_k
         Py_ssize_t pos = 0;
 
         while (PyDict_Next(dict, &pos, &key, &value)) {
-            PyObject* encoded;
-            if (PyUnicode_CheckExact(key)) {
-                encoded = PyUnicode_AsUTF8String(key);
-                if (!encoded) {
-                    return 0;
-                }
-            } else {
-                encoded = key;
-                Py_INCREF(encoded);
-            }
-            /* Don't allow writing _id here - it was written above. */
-            if (!write_pair(buffer, PyString_AsString(encoded),
-                            PyString_Size(encoded), value, check_keys, 0)) {
-                Py_DECREF(encoded);
+            if (!decode_and_write_pair(buffer, key, value, check_keys)) {
                 return 0;
             }
         }
@@ -1030,33 +973,22 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
         {
             int size;
             memcpy(&size, buffer + *position, 4);
-            if (strcmp(buffer + *position + 5, "$ref") == 0) { /* DBRef */
-                char id_type;
-                PyObject* id;
-
-                int offset = *position + 14;
-                int collection_length = strlen(buffer + offset);
-                PyObject* collection = PyUnicode_DecodeUTF8(buffer + offset, collection_length, "strict");
-                if (!collection) {
-                    return NULL;
-                }
-                offset += collection_length + 1;
-                id_type = buffer[offset];
-                offset += 5;
-                id = get_value(buffer, &offset, (int)id_type);
-                if (!id) {
-                    Py_DECREF(collection);
-                    return NULL;
-                }
-                value = PyObject_CallFunctionObjArgs(DBRef, collection, id, NULL);
-                Py_DECREF(collection);
-                Py_DECREF(id);
-            } else {
-                value = elements_to_dict(buffer + *position + 4, size - 5);
-                if (!value) {
-                    return NULL;
-                }
+            value = elements_to_dict(buffer + *position + 4, size - 5);
+            if (!value) {
+                return NULL;
             }
+
+            /* Decoding for DBRefs */
+            if (strcmp(buffer + *position + 5, "$ref") == 0) { /* DBRef */
+                PyObject* id = PyDict_GetItemString(value, "$id");
+                PyObject* collection = PyDict_GetItemString(value, "$ref");
+                PyObject* database = PyDict_GetItemString(value, "$db");
+
+                /* This works even if there is no $db since database will be NULL and
+                   the call will be as if there were only two arguments specified. */
+                value = PyObject_CallFunctionObjArgs(DBRef, collection, id, database, NULL);
+            }
+
             *position += size;
             break;
         }
