@@ -16,6 +16,7 @@
 
 import types
 import warnings
+import struct
 
 import pymongo
 import bson
@@ -26,7 +27,6 @@ from errors import InvalidName, OperationFailure
 from code import Code
 
 _ZERO = "\x00\x00\x00\x00"
-_ONE = "\x01\x00\x00\x00"
 
 
 class Collection(object):
@@ -196,7 +196,7 @@ class Collection(object):
         return len(ids) == 1 and ids[0] or ids
 
     def update(self, spec, document,
-               upsert=False, manipulate=False, safe=False):
+               upsert=False, manipulate=False, safe=False, multi=False):
         """Update a single document in this collection.
 
         Raises TypeError if either spec or document isn't an instance of
@@ -209,13 +209,20 @@ class Collection(object):
           - `spec`: a SON object specifying elements which must be present for
             a document to be updated
           - `document`: a SON object specifying the document to be used for the
-            update or (in the case of an upsert) insert. For more information
-            on update syntax / modifiers see the MongoDB wiki_.
+            update or (in the case of an upsert) insert. See docs on MongoDB
+            `update modifiers <http://www.mongodb.org/display/DOCS/Updating>`_
           - `upsert` (optional): perform an upsert operation
           - `manipulate` (optional): manipulate the document before updating?
           - `safe` (optional): check that the update succeeded?
+          - `multi` (optional): update all documents that match `spec`, rather
+            than just the first matching document. The default value for
+            `multi` is currently False, but this might eventually change to
+            True. It is recommended that you specify this argument explicitly
+            for all update operations in order to prepare your code for that
+            change.
 
-        .. _wiki: http://www.mongodb.org/display/DOCS/Updating
+        .. versionadded:: 1.1.1
+           The `multi` parameter.
         """
         if not isinstance(spec, types.DictType):
             raise TypeError("spec must be an instance of dict")
@@ -227,7 +234,13 @@ class Collection(object):
         if upsert and manipulate:
             document = self.__database._fix_incoming(document, self)
 
-        message = upsert and _ONE or _ZERO
+        options = 0
+        if upsert:
+            options += 1
+        if multi:
+            options += 2
+
+        message = struct.pack("<i", options)
         message += bson.BSON.from_dict(spec)
         message += bson.BSON.from_dict(document)
 
@@ -342,14 +355,14 @@ class Collection(object):
           - `snapshot` (optional): if True, snapshot mode will be used for this
             query. Snapshot mode assures no duplicates are returned, or objects
             missed, which were present at both the start and end of the query's
-            execution. For details, see the `wiki
+            execution. For details, see the `snapshot documentation
             <http://www.mongodb.org/display/DOCS/How+to+do+Snapshotting+in+the+Mongo+Database>`_.
           - `tailable` (optional): the result of this find call will be a
             tailable cursor - tailable cursors aren't closed when the last data
             is retrieved but are kept open and the cursors location marks the
             final document's position. if more data is received iteration of
             the cursor will continue from the last document received. For
-            details, see the `wiki
+            details, see the `tailable cursor documentation
             <http://www.mongodb.org/display/DOCS/Tailable+Cursors>`_.
         """
         if spec is None:
@@ -674,6 +687,21 @@ class Collection(object):
 
         self.__database.connection().admin._command(rename_command)
 
+    def distinct(self, key):
+        """Get a list of distinct values for `key` among all documents in this
+        collection.
+
+        :Parameters:
+          - `key`: name of key for which we want to get the distinct values
+
+        .. versionadded:: 1.1.1
+        """
+        if not isinstance(key, types.StringTypes):
+            raise TypeError("key must be an instance of (str, unicode)")
+
+        command = SON([("distinct", self.__collection_name), ("key", key)])
+        return self.__database._command(command)["values"]
+
     def __iter__(self):
         return self
 
