@@ -29,53 +29,70 @@ gen_target = 100
 reduction_attempts = 10
 examples = 5
 
+
 def lift(value):
     return lambda: value
 
+
 def choose_lifted(generator_list):
     return lambda: random.choice(generator_list)
 
+
 def map(generator, function):
     return lambda: function(generator())
 
+
 def choose(list):
     return lambda: random.choice(list)()
 
+
 def gen_range(start, stop):
     return lambda: random.randint(start, stop)
 
+
 def gen_int():
     max_int = 2147483647
     return lambda: random.randint(-max_int - 1, max_int)
 
+
 def gen_float():
     return lambda: (random.random() - 0.5) * sys.maxint
 
+
 def gen_boolean():
     return lambda: random.choice([True, False])
 
+
 def gen_printable_char():
     return lambda: chr(random.randint(32, 126))
 
+
 def gen_printable_string(gen_length):
     return lambda: "".join(gen_list(gen_printable_char(), gen_length)())
 
+
 def gen_char(set=None):
     return lambda: chr(random.randint(0, 255))
 
+
 def gen_string(gen_length):
     return lambda: "".join(gen_list(gen_char(), gen_length)())
 
+
 def gen_unichar():
     return lambda: unichr(random.randint(1, 0xFFF))
 
+
 def gen_unicode(gen_length):
-    return lambda: u"".join([x for x in gen_list(gen_unichar(), gen_length)() if x not in ".$"])
+    return lambda: u"".join([x for x in
+                             gen_list(gen_unichar(), gen_length)() if
+                             x not in ".$"])
+
 
 def gen_list(generator, gen_length):
     return lambda: [generator() for _ in range(gen_length())]
 
-# This is a little Mongo specific...
+
 def gen_datetime():
     return lambda: datetime.datetime(random.randint(1970, 2037),
                                      random.randint(1, 12),
@@ -85,7 +102,9 @@ def gen_datetime():
                                      random.randint(0, 59),
                                      random.randint(0, 999) * 1000)
 
+
 def gen_dict(gen_key, gen_value, gen_length):
+
     def a_dict(gen_key, gen_value, length):
         result = {}
         for _ in range(length):
@@ -93,11 +112,13 @@ def gen_dict(gen_key, gen_value, gen_length):
         return result
     return lambda: a_dict(gen_key, gen_value, gen_length())
 
+
 def gen_regexp(gen_length):
     # TODO our patterns only consist of one letter.
-    # this is because of a bug in CPython's regex equality testing, which I haven't
-    # quite tracked down, so I'm just ignoring it...
+    # this is because of a bug in CPython's regex equality testing,
+    # which I haven't quite tracked down, so I'm just ignoring it...
     pattern = lambda: u"".join(gen_list(choose_lifted(u"a"), gen_length)())
+
     def gen_flags():
         flags = 0
         if random.random() > 0.5:
@@ -110,13 +131,16 @@ def gen_regexp(gen_length):
         return flags
     return lambda: re.compile(pattern(), gen_flags())
 
+
 def gen_objectid():
     return lambda: ObjectId()
 
+
 def gen_dbref():
     collection = gen_unicode(gen_range(0, 20))
     return lambda: DBRef(collection(), gen_mongo_value(1, True)())
 
+
 def gen_mongo_value(depth, ref):
     choices = [gen_unicode(gen_range(0, 50)),
                gen_printable_string(gen_range(0, 50)),
@@ -126,7 +150,7 @@ def gen_mongo_value(depth, ref):
                gen_boolean(),
                gen_datetime(),
                gen_objectid(),
-               lift(None),]
+               lift(None)]
     if ref:
         choices.append(gen_dbref())
     if depth > 0:
@@ -134,14 +158,18 @@ def gen_mongo_value(depth, ref):
         choices.append(gen_mongo_dict(depth, ref))
     return choose(choices)
 
+
 def gen_mongo_list(depth, ref):
     return gen_list(gen_mongo_value(depth - 1, ref), gen_range(0, 10))
 
-def gen_mongo_dict(depth, ref=True):
-    return map(gen_dict(gen_unicode(gen_range(0, 20)), gen_mongo_value(depth - 1, ref), gen_range(0, 10)), SON)
 
-# TODO this is a hack - only really works for mongo_dicts right now...
-def simplify(case):
+def gen_mongo_dict(depth, ref=True):
+    return map(gen_dict(gen_unicode(gen_range(0, 20)),
+                        gen_mongo_value(depth - 1, ref),
+                        gen_range(0, 10)), SON)
+
+
+def simplify(case): # TODO this is a hack
     if isinstance(case, SON) and "$ref" not in case:
         simplified = SON(case) # make a copy!
         if random.choice([True, False]):
@@ -175,6 +203,8 @@ def simplify(case):
             simplified[index] = value
             return (success, success and simplified or case)
     return (False, case)
+
+
 def reduce(case, predicate, reductions=0):
     for _ in range(reduction_attempts):
         (reduced, simplified) = simplify(case)
@@ -182,6 +212,7 @@ def reduce(case, predicate, reductions=0):
             return reduce(simplified, predicate, reductions + 1)
     return (reductions, case)
 
+
 def isnt(predicate):
     return lambda x: not predicate(x)
 
@@ -198,12 +229,13 @@ def check(predicate, generator):
             counter_examples.append("%r : %s" % (case, traceback.format_exc()))
     return counter_examples
 
+
 def check_unittest(test, predicate, generator):
     counter_examples = check(predicate, generator)
     if counter_examples:
         failures = len(counter_examples)
-        message = "\n".join(["    -> %s" % f for f in counter_examples[:examples]])
-        message = "found %d counter examples, displaying first %d:\n%s" % (failures,
-                                                                          min(failures, examples),
-                                                                          message)
+        message = "\n".join(["    -> %s" % f for f in
+                             counter_examples[:examples]])
+        message = ("found %d counter examples, displaying first %d:\n%s" %
+                   (failures, min(failures, examples), message))
         test.fail(message)
