@@ -166,6 +166,8 @@ class TestBSON(unittest.TestCase):
         helper({"another binary": Binary("test")})
         helper(SON([(u'test dst', datetime.datetime(1993, 4, 4, 2))]))
         helper({"big float": float(10000000000)})
+        helper({"ref": DBRef("coll", 5)})
+        helper({"ref": DBRef("coll", 5, "foo")})
 
         def from_then_to_dict(dict):
             return dict == (BSON.from_dict(dict)).to_dict()
@@ -232,6 +234,15 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(id, transformed_id)
         self.assertNotEqual(uuid.uuid4(), transformed_id)
 
+    # The C extension was segfaulting on unicode RegExs, so we have this test
+    # that doesn't really test anything but the lack of a segfault.
+    def test_unicode_regex(self):
+        regex = re.compile(u'revisi\xf3n')
+        BSON.from_dict({"regex": regex}).to_dict()
+
+    def test_non_string_keys(self):
+        self.assertRaises(InvalidDocument, BSON.from_dict, {8.9: "test"})
+
 # TODO this test doesn't pass w/ C extension
 #
 # timegm doesn't handle years < 1900 (negative), at least on OS X
