@@ -20,6 +20,8 @@ import time
 import sys
 sys.path[0:0] = [""]
 
+from nose.plugins.skip import SkipTest
+
 import qcheck
 from test_connection import get_connection
 import version
@@ -489,6 +491,23 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(db.test.find_one(id1)["x"], 7)
         self.assertEqual(db.test.find_one(id2)["x"], 1)
 
+    def test_multi_update(self):
+        db = self.db
+        if not version.at_least(db.connection(), (1, 1, 3, -1)):
+            raise SkipTest()
+
+        db.drop_collection("test")
+
+        db.test.save({"x": 4, "y": 3})
+        db.test.save({"x": 5, "y": 5})
+        db.test.save({"x": 4, "y": 4})
+
+        db.test.update({"x": 4}, {"$set": {"y": 5}}, multi=True)
+
+        self.assertEqual(3, db.test.count())
+        for doc in db.test.find():
+            self.assertEqual(5, doc["y"])
+
     def test_upsert(self):
         db = self.db
         db.drop_collection("test")
@@ -501,7 +520,7 @@ class TestCollection(unittest.TestCase):
 
     def test_safe_update(self):
         db = self.db
-        v11 = version.at_least(db.connection(), (1, 1))
+        v113minus = version.at_least(db.connection(), (1, 1, 3, -1))
 
         db.drop_collection("test")
         db.test.create_index("x", unique=True)
@@ -510,7 +529,7 @@ class TestCollection(unittest.TestCase):
         id = db.test.insert({"x": 4})
 
         db.test.update({"_id": id}, {"$inc": {"x": 1}})
-        if v11:
+        if v113minus:
             self.assert_(db.error()["err"].startswith("E11001"))
         else:
             self.assert_(db.error()["err"].startswith("E12011"))
@@ -540,8 +559,10 @@ class TestCollection(unittest.TestCase):
         db.test.remove({"x": 1})
         self.assertEqual(1, db.test.count())
 
-        # TODO test this raises an exception (waiting on db support for reporting this)
-        db.test.remove({"x": 1}, safe=True)
+        if version.at_least(db.connection(), (1, 1, 3, -1)):
+            self.assertRaises(OperationFailure, db.test.remove, {"x": 1}, safe=True)
+        else: # Just test that it doesn't blow up
+            db.test.remove({"x": 1}, safe=True)
 
     def test_count(self):
         db = self.db
@@ -770,6 +791,35 @@ class TestCollection(unittest.TestCase):
         list(self.db.test.find(timeout=False))
         list(self.db.test.find(timeout=True))
 
+    def test_distinct(self):
+        if not version.at_least(self.db.connection(), (1, 1)):
+            raise SkipTest()
+
+        self.db.drop_collection("test")
+
+        self.db.test.save({"a": 1})
+        self.db.test.save({"a": 2})
+        self.db.test.save({"a": 2})
+        self.db.test.save({"a": 2})
+        self.db.test.save({"a": 3})
+
+        distinct = self.db.test.distinct("a")
+        distinct.sort()
+
+        self.assertEqual([1, 2, 3], distinct)
+
+        self.db.drop_collection("test")
+
+        self.db.test.save({"a": {"b": "a"}, "c": 12})
+        self.db.test.save({"a": {"b": "b"}, "c": 12})
+        self.db.test.save({"a": {"b": "c"}, "c": 12})
+        self.db.test.save({"a": {"b": "c"}, "c": 12})
+
+        distinct = self.db.test.distinct("a.b")
+        distinct.sort()
+
+        self.assertEqual(["a", "b", "c"], distinct)
+
 
 if __name__ == "__main__":
     unittest.main()
